#!/usr/bin/python
# -*- coding:utf-8 -*-

# Codé par 20-100

""" Module de Basile pour dialoguer avec la NoteKfet2015 """

import socket
import json
import ssl
import traceback

#: Config de basile
import config

class NKError(Exception):
    def __init__(self,msg):
        Exception.__init__(self)
        self.msg = msg
    def __str__(self):
        return str(self.msg)
    def __unicode__(self):
        return unicode(self.msg)

class NKRefused(NKError):
    pass

class NKHelloFailed(NKError):
    pass

class NKUnknownError(NKError):
    pass

class NKDeadServer(NKError):
    pass

def full_read(sock):
    """Lit sur la socket jusqu'à ce que l'output soit déJSON-izable"""
    output = ""
    i = 0
    while True:
        output += sock.read()
        i += 1
        try:
            return json.loads(output)
        except:
            pass
        if i == 10 and output == "":
            raise NKDeadServer

def connect():
    sock = socket.socket()
    try:
        # On établit la connexion sur port 4242
        sock.connect((config.nk_server, config.nk_port))
        # On passe en SSL
        sock = ssl.wrap_socket(sock, ca_certs='../keys/ca_.crt')
        # On fait un hello
        sock.write(json.dumps(["hello", "Basile"]))
        # On récupère la réponse du hello
        out = full_read(sock)
    except Exception as exc:
        # Si on a foiré quelque part, c'est que le serveur est down
        raise NKRefused(str(exc))
    if out["retcode"] == 0:
        return sock
    elif out["retcode"] == 11:
        raise NKHelloFailed(out["errmsg"])
    else:
        raise NKUnknownError(out["errmsg"])


def login(username, password, typ="bdd"):
    sock = connect()
    if typ == "special": # ça c'est pour Basile lui-même
        masque = []
    elif typ == "bdd":
        masque = [[], [], True]
    try:
        # Basile a un compte special user
        commande = ["login", [username, password, typ, masque]]
        sock.write(json.dumps(commande))
        out = full_read(sock)
    except Exception as exc:
        # Si on a foiré quelque part, c'est que le serveur est down
        raise NKRefused(str(exc))
    return out, sock

def get_solde(sock, idbde, serv, canal):
    """Récupère le (success, solde, pseudo) de l'utilisateur NK n°``idbde``"""
    try:
        sock.write(json.dumps(["compte", idbde]))
        ret = full_read(sock)
        retcode = ret["retcode"]
        if retcode == 0:
            solde = ret["msg"]["solde"]
            pseudo = ret["msg"]["pseudo"]
            return (True, solde, pseudo)
        else:
            serv.privmsg(canal, ret["errmsg"])
            return (False, None, None)
    except Exception as exc:
        trace = traceback.format_exc()
        msg = "failed\n%s" % trace
        for l in msg.split("\n"):
            serv.privmsg(canal, l)
        #log(self.serveur, "priv", auteur, " ".join(message) + "[failed]")
        return (False, None, None)

def consomme(sock, idbde, conso, serv, canal):
    """Fais consommer une conso à l'utilisateur NK n°``idbde``"""
    try:
        sock.write(json.dumps(["get_boutons", ["", ""]]))
        ret = full_read(sock)
        retcode = ret["retcode"]
        if retcode == 0:
            boutons = ret["msg"]
            boutons = [b for b in boutons if b["label"].lower() == conso.lower()]
            if len(boutons) == 0:
                serv.privmsg(canal, (u"Impossible de trouver la conso %s" % (conso)).encode("utf-8"))
                return False
            bouton = boutons[0]
            sock.write(json.dumps(["consos", [[bouton["id"], idbde, 1]]]))
            ret = full_read(sock)
            if ret["retcode"] == 0:
                [[retcode, [idbouton, idbde], errmsg]] = ret["msg"]
                if retcode != 0:
                    serv.privmsg(canal, errmsg.encode("utf-8"))
                else:
                    success, solde, pseudo = get_solde(sock, idbde, serv, canal)
                    if success:
                        serv.privmsg(canal, (u"%s consomme 1 %s (nouveau solde : %.2f)" % (pseudo, bouton["label"], solde/100.0)).encode("utf-8"))
                    else:
                        serv.privmsg(canal, (u"%s consommé mais impossible de récupérer le solde après transaction." % (bouton["label"])).encode("utf-8"))
                    return True # on a réussi à consommer la conso
            else:
                serv.privmsg(canal, ret["errmsg"].encode("utf-8"))
                return False
        else:
            serv.privmsg(canal, ret["errmsg"])
    except Exception as exc:
        trace = traceback.format_exc()
        msg = "failed\n%s" % trace
        for l in msg.split("\n"):
            serv.privmsg(canal, l)

